@@ -2,18 +2,20 @@
 """peutils, Portable Executable utilities module
 
 
-Copyright (c) 2005-2012 Ero Carrera <ero.carrera@gmail.com>
+Copyright (c) 2005-2013 Ero Carrera <ero.carrera@gmail.com>
 
 All rights reserved.
-
-For detailed copyright information see the file COPYING in
-the root of the distribution archive.
 """
+from __future__ import division
+from future import standard_library
+standard_library.install_aliases()
+from builtins import range
+from builtins import object
 
 import os
 import re
 import string
-import urllib
+import urllib.request, urllib.parse, urllib.error
 import pefile
 
 __author__ = 'Ero Carrera'
@@ -23,7 +25,7 @@ __contact__ = 'ero.carrera@gmail.com'
 
 
 
-class SignatureDatabase:
+class SignatureDatabase(object):
     """This class loads and keeps a parsed PEiD signature database.
 
     Usage:
@@ -195,7 +197,7 @@ class SignatureDatabase:
             #
             try :
                 data = pe.__data__
-            except Exception, excp :
+            except Exception as excp :
                 raise
 
             # Load the corresponding tree of signatures
@@ -213,7 +215,7 @@ class SignatureDatabase:
             #
             try :
                 data = pe.get_memory_mapped_image()
-            except Exception, excp :
+            except Exception as excp :
                 raise
 
             # Load the corresponding tree of signatures
@@ -235,7 +237,7 @@ class SignatureDatabase:
 
             signatures = self.signature_tree_eponly_false
 
-            scan_addresses = xrange( len(data) )
+            scan_addresses = range( len(data) )
 
         # For each start address, check if any signature matches
         #
@@ -319,7 +321,7 @@ class SignatureDatabase:
         # Walk the bytes in the data and match them
         # against the signature
         #
-        for idx, byte in enumerate ( [ord (b) for b in data] ):
+        for idx, byte in enumerate ( [b if isinstance(b, int) else ord(b) for b in data] ):
 
             # If the tree is exhausted...
             #
@@ -335,7 +337,7 @@ class SignatureDatabase:
             # it means that a signature in the database
             # ends here and that there's an exact match.
             #
-            if None in match.values():
+            if None in list(match.values()):
                 # idx represent how deep we are in the tree
                 #
                 #names = [idx+depth]
@@ -345,7 +347,7 @@ class SignatureDatabase:
                 # if it has an element other than None,
                 # if not then we have an exact signature
                 #
-                for item in match.items():
+                for item in list(match.items()):
                     if item[1] is None :
                         names.append (item[0])
                 matched_names.append(names)
@@ -353,7 +355,7 @@ class SignatureDatabase:
             # If a wildcard is found keep scanning the signature
             # ignoring the byte.
             #
-            if match.has_key ('??') :
+            if '??' in match :
                 match_tree_alternate = match.get ('??', None)
                 data_remaining = data[idx + 1 :]
                 if data_remaining:
@@ -366,10 +368,10 @@ class SignatureDatabase:
         # If we have any more packer name in the end of the signature tree
         # add them to the matches
         #
-        if match is not None and None in match.values():
+        if match is not None and None in list(match.values()):
             #names = [idx + depth + 1]
             names = list()
-            for item in match.items() :
+            for item in list(match.items()) :
                 if item[1] is None:
                     names.append(item[0])
             matched_names.append(names)
@@ -392,7 +394,7 @@ class SignatureDatabase:
             #
             if not os.path.exists(filename):
                 try:
-                    sig_f = urllib.urlopen(filename)
+                    sig_f = urllib.request.urlopen(filename)
                     sig_data = sig_f.read()
                     sig_f.close()
                 except IOError:
@@ -402,7 +404,7 @@ class SignatureDatabase:
                 # Get the data for a file
                 #
                 try:
-                    sig_f = file( filename, 'rt' )
+                    sig_f = open( filename, 'rt' )
                     sig_data = sig_f.read()
                     sig_f.close()
                 except IOError:
@@ -419,10 +421,10 @@ class SignatureDatabase:
 
         # Helper function to parse the signature bytes
         #
-        def to_byte(value) :
-            if value == '??' or value == '?0' :
+        def to_byte(value):
+            if '?' in value:
                 return value
-            return int (value, 16)
+            return int(value, 16)
 
 
         # Parse all the signatures in the file
@@ -558,6 +560,9 @@ def is_probably_packed( pe ):
     # file. Overlay data won't be taken into account
     #
     total_pe_data_length = len( pe.trim() )
+    # Assume that the file is packed when no data is available
+    if not total_pe_data_length:
+        return True
     has_significant_amount_of_compressed_data = False
 
     # If some of the sections have high entropy and they make for more than 20% of the file's size
@@ -576,6 +581,3 @@ def is_probably_packed( pe ):
         has_significant_amount_of_compressed_data = True
 
     return has_significant_amount_of_compressed_data
-
-
-
